/**
 * @file annotation.cpp
 * @author Lars Johannesen <lj@halloffame.dk>
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * This file has the annotation class definition
 */
#include "ecgdata.hpp"

#include "annotation.hpp"

namespace ecglib {
	void get_annotations(const ecgdata::annotationset &pm, const ecglib::annotation_type &typ, std::vector<annotation> &out) {
		typedef ecglib::ecgdata::const_pointiterator piter;
		piter pend = pm.end();
		for(piter pi = pm.begin(); pi != pend; ++pi) {
			if(pi->second.type() == typ) {
				out.push_back(pi->second);
			}
		}
	}

	void get_annotations(const ecgdata::pointmap &pm, const ecglib::annotation_type &typ, std::vector<annotation> &out) {
		typedef ecglib::ecgdata::const_pointmapiterator pmiter;

		pmiter pmend = pm.end();

		for(pmiter pmi = pm.begin(); pmi != pmend; ++pmi) {
			typedef ecglib::ecgdata::const_pointiterator piter;

			piter pend = pmi->second.end();
			for(piter pi = pmi->second.begin(); pi != pend; ++pi) {
				if(pi->second.type() == typ) {
					out.push_back(pi->second);
				}
			}
		}
	}

	ecgdata::annotationset rebase(const ecgdata::annotationset &ann, const int newstart) {
		typedef ecgdata::const_pointiterator piter;

		piter pend = ann.end();

		ecgdata::annotationset rebased;

		for(piter pi = ann.begin(); pi != pend; ++pi) {
			ecglib::annotation newann = pi->second;
			newann.location(newann.location()-newstart);

			rebased[newann.location()] = newann;
		}

		return rebased;
	}

	void rebase(ecgdata::pointmap &pm, const int newstart) {
		typedef ecglib::ecgdata::pointmapiterator pmiter;
	
		pmiter pmend = pm.end();
		for(pmiter pmi = pm.begin(); pmi != pmend; ++pmi) {
			pmi->second = rebase(pmi->second, newstart);
		}
	}
}
