/**
 * @file rrpredictor.cpp
 * @author Lars Johannesen <lj@halloffame.dk>
 *
 * @section LICENSE
 * This program is licensed under LGPL3, see LICENSE
 * 
 * @section DESCRIPTION
 * The classes in this file are used for describing characteristics about ecg records
 */

#include <ecglib_bits/qrsdetectors/rrpredictor.hpp>
#include <ecglib_bits/qrsdetectors/qrsdetector.hpp>

namespace ecglib { 
	rrpredictor::rrpredictor(const qrsdetector_config &cfg) : _ready(false), _first(true), _missed(0) {
		_nrr = cfg.get<unsigned int>("rrhist");
		_rrhist.set_capacity(_nrr);
	}

	void rrpredictor::reset() {
		_rrhist.clear();
		_rrhist.set_capacity(_nrr);

		_ready = false;
		_first = true;
		_missed = 0;
	}

	void rrpredictor::detectqrs(const unsigned int loc, const double fs) {
		unsigned int nloc = static_cast<unsigned int>(round(loc * 1000./fs));

		if (_ready) {
			unsigned int tmprr = nloc - _lastqrs;

			if (tmprr > 0.8*_meanrr && tmprr < 1.6*_meanrr) {
				_rrhist.push_back(nloc-_lastqrs);
			}

			_lastqrs = nloc;
		} else {
			if(_first) {
				_lastqrs = nloc;
				_first = false;
			} else {
				_rrhist.push_back(nloc-_lastqrs);
				_lastqrs = nloc;
			}

			if(_rrhist.size() == _nrr) _ready = true;
		}

		double tmp=0;
		for(std::size_t i = 0; i < _rrhist.size(); ++i) {
			tmp += _rrhist[i]/(_rrhist.size()*1.0);
		}
		_meanrr = static_cast<unsigned int>(round(tmp));
		_nextqrs = nloc + 1.66 * _meanrr;
		_missed = 0;
	}
}
