/********************************************************************************
 *                                  DISCLAIMER                                  *
 * This software and documentation (the "Software") were developed at the Food  *
 * and Drug Administration (FDA) by employees of the Federal Government in the  *
 * course of their official duties. Pursuant to Title 17, Section 105 of the    *
 * United States Code, this work is not subject to copyright protection and is  *
 * in the public domain. Permission is hereby granted, free of charge, to any   *
 * person obtaining a copy of the Software, to deal in the Software without     *
 * restriction, including without limitation the rights to use, copy, modify,   *
 * merge, publish, distribute, sublicense, or sell copies of the Software or    *
 * derivatives, and to permit persons to whom the Software is furnished to do   *
 * so. FDA assumes no responsibility whatsoever for use by other parties of     *
 * the Software, its source code, documentation or compiled executables, and    *
 * makes no guarantees, expressed or implied, about its quality, reliability,   *
 *  or any other characteristic. Further, use of this code in no way implies    *
 * endorsement by the FDA or confers any advantage in regulatory decisions.     *
 * Although this software can be redistributed and/or modified freely, we       *
 * ask that any derivative works bear some notice that they are derived from    *
 * it, and any modified versions bear some notice that they have been modified. * 
 *                                                                              *
 * Written by:                                                                  *
 *  Lars Johannesen                                                             *
********************************************************************************/
#include "mex.h"

#include <stdexcept>
#include <cstdlib>

#include <armadillo>
using namespace arma;

template<typename TYP>
mxArray *set_matlab_scalar(TYP t, mxClassID cla=mxDOUBLE_CLASS) {
        mwSize dims[2] = {1,1};
        mxArray *o = mxCreateNumericArray(2, dims, cla, mxREAL);
        double *d = (double*)mxGetData(o);
        *d=t;
        return o;
}

mat get_mat(const mxArray *arg) {
	double *ptr = (double*)mxGetPr(arg);
	int cols = mxGetN(arg);
	int rows = mxGetM(arg);

	mat omat(ptr, rows, cols);

	return omat;
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[])
{
	if(nrhs != 3)
		mexErrMsgTxt("Need 3 arguments: annmat, sqi, window");

	try {
		if(!mxIsDouble(prhs[0]))
			mexErrMsgTxt("First argument was not a double");

		if(!mxIsDouble(prhs[1]))
			mexErrMsgTxt("Second argument was not a double");

		if(!mxIsDouble(prhs[2]))
			mexErrMsgTxt("Third argument was not a double scalar");

		mat ann = get_mat(prhs[0]);
		if(ann.n_cols != 1) {
			mexErrMsgTxt("First argument must be nx1");
		}

		mat sqi = get_mat(prhs[1]);
		if(sqi.n_cols != 1) {
			mexErrMsgTxt("Second argument must be nx1");
		}

		if(sqi.n_rows != ann.n_rows) {
			mexErrMsgTxt("First and Second argument must have same number of rows");
		}
		
		double *ptr = (double*)mxGetPr(prhs[2]);
		double threshold = *ptr;

		// Store annots
		mxArray *mmat = mxCreateDoubleMatrix(ann.n_rows,1,mxREAL);
		double *d = (double*) mxGetData(mmat);
		mat omat(d, ann.n_rows, 1, false, true);
		omat.fill(1);

		for(std::size_t i = 0; i < sqi.n_rows; ++i) {
			if(sqi(i) == 0) {
				double curtime = ann(i);
				omat(i) = 0;

				for(std::size_t j = 0; j < ann.n_rows; ++j) {
					if(omat(j) == 0) continue;

					if(fabs(curtime - ann(j)) < threshold) {
						omat(j) = 0.5;
					}
				}

			}
		}

		plhs[0] = mmat;
	} catch (std::logic_error &exp) {
		mexErrMsgTxt(exp.what());
	} catch (...) {
		mexErrMsgTxt("Unknown error");
	}
}
